@@ -177,6 +177,7 @@ static void adapt_window_size (EogWindow *window, int width, int height);
 #endif
 static void update_status_bar (EogWindow *window);
 static void job_default_progress (EogJob *job, gpointer data, float progress);
+static void add_uri_to_recent_files (EogWindow *window, GnomeVFSURI *uri);
 
 
 static GtkWindowClass *parent_class;
@@ -791,6 +792,7 @@ typedef struct {
 typedef struct {
 	EogWindow   *window;
 	EogImage    *image;
+	EogImageSaveInfo *dest;
 	
 	GError      *error;
 	EogSaveResponse response;
@@ -870,7 +872,9 @@ save_error (SaveErrorData *edata)
 	if (err_code == EOG_IMAGE_ERROR_FILE_EXISTS) {
 		char *str;
 
-		str = eog_image_get_uri_for_display (edata->image); 
+	       	str = gnome_vfs_uri_to_string (edata->dest->uri, 
+					       GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+		
 		header = g_strdup_printf (_("Overwrite file %s?"), str);
 		detail = _("File exists. Do you want to overwrite it?");
 		g_free (str);
@@ -923,12 +927,20 @@ static void
 job_save_image_finished (EogJob *job, gpointer user_data, GError *error)
 {
 	SaveData *data = (SaveData*) user_data;
-
+	
 	eog_save_dialog_close (data->dlg, !data->cancel_save);
 
 	/* enable image modification functions */
 	gtk_action_group_set_sensitive (data->window->priv->actions_image,  TRUE);
 
+	/* Only set window title on "Save As" with a single image */
+	g_mutex_lock (data->lock);
+	if (data->dest != NULL && data->n_images == 1) {
+		g_print ("SET DONE\n");
+		gtk_window_set_title (GTK_WINDOW (data->window), 
+				      eog_image_get_caption (EOG_IMAGE (data->images->data)));
+	}
+	g_mutex_unlock (data->lock);
 }
 
 static SaveData*
@@ -1010,7 +1022,7 @@ job_save_image_progress (EogJob *job, gpointer user_data, float progress)
 }
 
 static SaveErrorData*
-save_error_data_new (GError *error, EogWindow *window, EogImage *image)
+save_error_data_new (GError *error, EogWindow *window, EogImage *image, EogImageSaveInfo *dest)
 {
 	SaveErrorData *edata;
 
@@ -1024,6 +1036,7 @@ save_error_data_new (GError *error, EogWindow *window, EogImage *image)
 	edata->lock     = g_mutex_new ();
 	edata->window   = window;
 	edata->image    = g_object_ref (image);
+	edata->dest     = dest;
 
 	if (error != NULL) {
 		edata->error    = g_error_copy (error); 
@@ -1053,7 +1066,7 @@ job_save_handle_error (SaveData *data, EogImage *image, GError *error)
 	SaveErrorData *edata;
 	gboolean success = FALSE;
 
-	edata = save_error_data_new (error, data->window, image);
+	edata = save_error_data_new (error, data->window, image, data->dest);
 	
 	g_mutex_lock (edata->lock);
 	g_idle_add ((GSourceFunc) save_error, edata);
@@ -1986,7 +1999,7 @@ verb_Delete_cb (GtkAction *action, gpointer data)
 	images = eog_wrap_list_get_selected_images (EOG_WRAP_LIST (priv->wraplist));	
 	g_assert (images != NULL);
 	pos = eog_image_list_get_pos_by_img (list, EOG_IMAGE (images->data));
-
+	
 	/* FIXME: make a nice progress dialog */
 	/* Do the work actually. First try to delete the image from the disk. If this
 	 * is successfull, remove it from the screen. Otherwise show error dialog.
@@ -2028,12 +2041,15 @@ verb_Delete_cb (GtkAction *action, gpointer data)
 	g_list_free (images);
 
 	/* select image at previously saved position */
-	pos = MIN (pos, eog_image_list_length (list));
-	img = eog_image_list_get_img_by_pos (list, pos);
+	pos = MIN (pos, eog_image_list_length (list) - 1);
 
-	eog_wrap_list_set_current_image (EOG_WRAP_LIST (priv->wraplist), img, TRUE);
-	if (img != NULL) {
-		g_object_unref (img);
+	if (pos >= 0) {
+		img = eog_image_list_get_img_by_pos (list, pos);
+
+		eog_wrap_list_set_current_image (EOG_WRAP_LIST (priv->wraplist), img, TRUE);
+		if (img != NULL) {
+			g_object_unref (img);
+		}
 	}
 }
 
@@ -2885,7 +2901,6 @@ display_image_data (EogWindow *window, EogImage *image)
 
 	eog_scroll_view_set_image (EOG_SCROLL_VIEW (priv->scroll_view), image);
 	eog_info_view_set_image (EOG_INFO_VIEW (priv->info_view), image);
-	update_status_bar (window);
 
 	if (priv->displayed_image != NULL)
 		g_object_unref (priv->displayed_image);
@@ -2898,7 +2913,12 @@ display_image_data (EogWindow *window, EogImage *image)
 		title = _("Eye of GNOME");		
 	}
 
-	gtk_window_set_title (GTK_WINDOW (window), title);		
+	gtk_window_set_title (GTK_WINDOW (window), title);
+
+	update_status_bar (window);
+
+	/* update recent files */
+	add_uri_to_recent_files (window, eog_image_get_uri (image));
 }
 
 /* this runs in its own thread */
@@ -2987,6 +3007,9 @@ handle_image_selection_changed (EogWrapList *list, EogWindow *window)
 
 	priv = window->priv;
 
+	if (eog_wrap_list_get_n_selected (EOG_WRAP_LIST (priv->wraplist)) == 0)
+		return;
+
 	image = eog_wrap_list_get_first_selected_image (EOG_WRAP_LIST (priv->wraplist));
 	g_assert (EOG_IS_IMAGE (image));
 
@@ -3270,6 +3293,8 @@ eog_window_construct_ui (EogWindow *window, GError **error)
 	GtkWidget *frame;
 	char *filename;
 	guint merge_id;
+	GtkAdjustment *vadj;
+	GtkAdjustment *hadj;
 
 	g_return_val_if_fail (window != NULL, FALSE);
 	g_return_val_if_fail (EOG_IS_WINDOW (window), FALSE);
@@ -3389,8 +3414,13 @@ eog_window_construct_ui (EogWindow *window, GError **error)
 	g_signal_connect (G_OBJECT (priv->wraplist), "selection_changed",
 			  G_CALLBACK (handle_image_selection_changed), window);
 
-	sw = gtk_scrolled_window_new (NULL, NULL);
+	vadj = gtk_adjustment_new (0, 100, 0, 10, 10, 100);
+	hadj = gtk_adjustment_new (0, 100, 0, 10, 10, 100);
+
+	sw = gtk_scrolled_window_new (hadj, vadj);
+	
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_IN);
+	
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
@@ -3575,6 +3605,7 @@ adapt_window_size (EogWindow *window, int width, int height)
 	}
 #endif
 }
+#endif
 
 static void
 add_uri_to_recent_files (EogWindow *window, GnomeVFSURI *uri)
@@ -3596,7 +3627,6 @@ add_uri_to_recent_files (EogWindow *window, GnomeVFSURI *uri)
 
 	g_free (text_uri);
 }
-#endif
 
 /**
  * eog_window_open:
@@ -3653,11 +3683,6 @@ eog_window_open (EogWindow *window, EogImageList *model, GError **error)
 	/* attach model to view */
 	eog_wrap_list_set_model (EOG_WRAP_LIST (priv->wraplist), EOG_IMAGE_LIST (priv->image_list));
 	
-	/* update recent files */
-#if 0
-	add_uri_to_recent_files (window, uri);
-#endif
-
 	return TRUE;
 }
 
